import os
from collections import OrderedDict
from functools import cached_property
from operator import index
from typing import Tuple

from garm import app, set_timezone
from garm.app.helpers import sql_yield
from garm.models import BaseUser
from opts import rabbit_port, rabbit_host, rabbit_login, rabbit_passwd, rabbit_login_method, rabbit_virtualhost, rabbit_heartbeat_timeout

from . import AbstractPokemon

__all__ = ['AbstractSqlPokemon', 'AbstractGarmPokemon']

class AbstractGarmPokemon(AbstractPokemon):
    _rabbit_host = rabbit_host
    _rabbit_port = rabbit_port
    _rabbit_login = rabbit_login
    _rabbit_passwd = rabbit_passwd
    _login_method = rabbit_login_method
    _virtualhost = rabbit_virtualhost
    _heartbeat_timeout = rabbit_heartbeat_timeout

    def prepare_server(self, user=None):
        app.user = user or BaseUser(user_uuid=self.__class__.__name__)

        return super().prepare_server(user)


class AbstractSqlPokemon(AbstractGarmPokemon):
    def __init__(self, uze_tz=None, prefetch_count=1, respond_to: 'AbstractPokemon' = None, **kwargs):
        super().__init__(prefetch_count=prefetch_count, respond_to=respond_to, **kwargs)
        self.tz = uze_tz

    async def execute(self, func, json):
        multi_thread = self._prefetch_count > 1
        with sql_yield(code=self.__class__.__name__, glob=not multi_thread) as s:
            set_timezone()

            args, kwargs = self._get_arg_kwarg(json)

            if multi_thread:
                if 's' in kwargs:
                    raise TypeError(
                        "Pokemon %s got an unexpected keyword argument 's'" % func)

                kwargs['s'] = s

            return await func(*args, **kwargs)
