import ujson

from aioamqp.channel import Channel
from aioamqp.envelope import Envelope
from aioamqp.properties import Properties

from . import AbstractPokemon, pokemon_method, check_session_reconnect


class AbstractPokeWitch(AbstractPokemon):
    need_klink = False

    incoming_exchange = 'TAP.Events'
    outgoing_exchange = 'TAP.Commands'
    outgoing_routing_prefix = 'command_'
    routing_key = '*.*.*.*.*'

    @pokemon_method
    @check_session_reconnect
    async def send_to_freeswitch(self, core_uuid: str, command: str) -> None:
        properties = {'delivery_mode': 2}
        self.logger.debug(f'Sending {core_uuid}....')
        await self.basic_publish(
            payload=command.encode(),
            exchange_name=self.outgoing_exchange,
            routing_key=f'{self.outgoing_routing_prefix}{core_uuid}',
            properties=properties,
        )
        self.logger.debug(f'Sent {core_uuid}....')

    async def received_event(self, data: dict) -> None:
        #  Получение и обработка полученных эвентов. Необходимо переопределение в классе.
        uuid = data['Core-UUID']
        event = data['Event-Name']

        await self.send_to_freeswitch(uuid, f'log {event} {uuid}')

    # noinspection PyUnusedLocal
    async def freeswitch_callback(self, channel: Channel, body: bytes, envelope: Envelope, properties: Properties) -> None:
        # noinspection TaskProblemsInspection,PyBroadException
        try:
            await self.received_event(data=ujson.loads(body))
        except:  # noqa: E733
            self.logger.exception('error')

    async def _receive(self) -> None:
        await super()._receive()

        name = f'{self._queue_name} freeswitch'

        await self.channel.queue_declare(
            queue_name=name,
            durable=True
        )

        await self.channel.queue_bind(
            queue_name=name,
            exchange_name=self.incoming_exchange,
            routing_key=self.routing_key
        )

        await self.channel.basic_consume(
            self.freeswitch_callback,
            queue_name=name,
            no_ack=True
        )

        self.logger.debug(f'Listening queue "{name}"')
